@@ -1,4 +1,4 @@
-# $Id: ODBC.pm 15193 2012-03-02 11:09:04Z mjevans $
+# $Id: ODBC.pm 15339 2012-06-26 10:13:40Z mjevans $
 #
 # Copyright (c) 1994,1995,1996,1998  Tim Bunce
 # portions Copyright (c) 1997-2004  Jeff Urlwin
@@ -19,7 +19,7 @@ require 5.008;
 # see discussion on dbi-users at
 # http://www.nntp.perl.org/group/perl.dbi.dev/2010/07/msg6096.html and
 # http://www.dagolden.com/index.php/369/version-numbers-should-be-boring/
-$DBD::ODBC::VERSION = '1.35';
+$DBD::ODBC::VERSION = '1.39';
 
 {
     ## no critic (ProhibitMagicNumbers ProhibitExplicitISA)
@@ -32,7 +32,7 @@ $DBD::ODBC::VERSION = '1.35';
 
     @ISA = qw(Exporter DynaLoader);
 
-    # my $Revision = substr(q$Id: ODBC.pm 15193 2012-03-02 11:09:04Z mjevans $, 13,2);
+    # my $Revision = substr(q$Id: ODBC.pm 15339 2012-06-26 10:13:40Z mjevans $, 13,2);
 
     require_version DBI 1.609;
 
@@ -60,12 +60,28 @@ $DBD::ODBC::VERSION = '1.35';
         SQL_DIAG_ROW_NUMBER => -1248,
         SQL_DIAG_SERVER_NAME => 11,
         SQL_DIAG_SQLSTATE => 4,
-        SQL_DIAG_SUBCLASS_ORIGIN => 9
+        SQL_DIAG_SUBCLASS_ORIGIN => 9,
+        # TAF constants - these are just copies of Oracle constants
+        # events:
+        OCI_FO_END     => 0x00000001,
+        OCI_FO_ABORT   => 0x00000002,
+        OCI_FO_REAUTH  => 0x00000004,
+        OCI_FO_BEGIN   => 0x00000008,
+        OCI_FO_ERROR   => 0x00000010,
+        # callback return codes:
+        OCI_FO_RETRY   => 25410,
+        # types:
+        OCI_FO_NONE    => 0x00000001,
+        OCI_FO_SESSION => 0x00000002,
+        OCI_FO_SELECT  => 0x00000004,
+        OCI_FO_TXNAL   => 0x00000008
     };
     our @EXPORT_DIAGS = qw(SQL_DIAG_CURSOR_ROW_COUNT SQL_DIAG_DYNAMIC_FUNCTION SQL_DIAG_DYNAMIC_FUNCTION_CODE SQL_DIAG_NUMBER SQL_DIAG_RETURNCODE SQL_DIAG_ROW_COUNT SQL_DIAG_CLASS_ORIGIN SQL_DIAG_COLUMN_NUMBER SQL_DIAG_CONNECTION_NAME SQL_DIAG_MESSAGE_TEXT SQL_DIAG_NATIVE SQL_DIAG_ROW_NUMBER SQL_DIAG_SERVER_NAME SQL_DIAG_SQLSTATE SQL_DIAG_SUBCLASS_ORIGIN);
-    our @EXPORT_OK = (@EXPORT_DIAGS);
+    our @EXPORT_TAF = qw(OCI_FO_END OCI_FO_ABORT OCI_FO_REAUTH OCI_FO_BEGIN OCI_FO_ERROR OCI_FO_RETRY OCI_FO_NONE OCI_FO_SESSION OCI_FO_SELECT OCI_FO_TXNAL);
+    our @EXPORT_OK = (@EXPORT_DIAGS, @EXPORT_TAF);
     our %EXPORT_TAGS = (
-        diags => \@EXPORT_DIAGS);
+        diags => \@EXPORT_DIAGS,
+        taf => \@EXPORT_TAF);
 
     sub parse_trace_flag {
         my ($class, $name) = @_;
@@ -177,44 +193,45 @@ $DBD::ODBC::VERSION = '1.35';
             odbc_putdata_start             => undef, # sth and dbh
             odbc_column_display_size       => undef, # sth and dbh
             odbc_utf8_on                   => undef, # sth and dbh
-	    odbc_driver_complete           => undef,
-	    odbc_batch_size                => undef,
-            odbc_disable_array_operations  => undef, # sth and dbh
-               };
+            odbc_driver_complete           => undef,
+            odbc_batch_size                      => undef,
+            odbc_array_operations          => undef, # sth and dbh
+            odbc_taf_callback                    => undef
+        };
     }
 
     sub prepare {
-	my($dbh, $statement, @attribs)= @_;
+        my($dbh, $statement, @attribs)= @_;
 
-	# create a 'blank' sth
-	my $sth = DBI::_new_sth($dbh, {
-	    'Statement' => $statement,
+        # create a 'blank' sth
+        my $sth = DBI::_new_sth($dbh, {
+            'Statement' => $statement,
 	    });
 
-	# Call ODBC func in ODBC.xs file.
-	# (This will actually also call SQLPrepare for you.)
-	# and populate internal handle data.
+        # Call ODBC func in ODBC.xs file.
+        # (This will actually also call SQLPrepare for you.)
+        # and populate internal handle data.
 
-	DBD::ODBC::st::_prepare($sth, $statement, @attribs)
-	    or return;
+        DBD::ODBC::st::_prepare($sth, $statement, @attribs)
+              or return;
 
-	return $sth;
+        return $sth;
     }
 
     sub column_info {
-	my ($dbh, $catalog, $schema, $table, $column) = @_;
+        my ($dbh, $catalog, $schema, $table, $column) = @_;
 
-	$catalog = q{} if (!$catalog);
-	$schema = q{} if (!$schema);
-	$table = q{} if (!$table);
-	$column = q{} if (!$column);
-	# create a "blank" statement handle
-	my $sth = DBI::_new_sth($dbh, { 'Statement' => "SQLColumns" });
+        $catalog = q{} if (!$catalog);
+        $schema = q{} if (!$schema);
+        $table = q{} if (!$table);
+        $column = q{} if (!$column);
+        # create a "blank" statement handle
+        my $sth = DBI::_new_sth($dbh, { 'Statement' => "SQLColumns" });
 
-	_columns($dbh,$sth, $catalog, $schema, $table, $column)
-	    or return;
+        _columns($dbh,$sth, $catalog, $schema, $table, $column)
+            or return;
 
-	return $sth;
+        return $sth;
     }
 
     sub columns {
@@ -536,8 +553,8 @@ $DBD::ODBC::VERSION = '1.35';
             odbc_exec_direct               => undef, # sth and dbh
             odbc_old_unicode               => undef, # sth and dbh
             odbc_describe_parameters       => undef, # sth and dbh
-	    odbc_batch_size                => undef, # sth and dbh
-            odbc_disable_array_operations  => undef, # sth and dbh
+            odbc_batch_size                => undef, # sth and dbh
+            odbc_array_operations          => undef, # sth and dbh
         };
     }
 
@@ -565,7 +582,10 @@ $DBD::ODBC::VERSION = '1.35';
                             ($tuple_status ? $tuple_status : 'undef') .
                                 ") batch_size = $batch_size\n", 4);
         # Use DBI's execute_for_fetch if ours is disabled
-        if ($sth->FETCH('odbc_disable_array_operations')) {
+        my $override = (defined($ENV{ODBC_DISABLE_ARRAY_OPERATIONS}) ?
+                            $ENV{ODBC_DISABLE_ARRAY_OPERATIONS} : -1);
+        if ((($sth->FETCH('odbc_array_operations') == 0) && ($override != 0)) ||
+                $override == 1) {
             $sth->trace_msg("array operations disabled\n", 4);
             my $sth = shift;
             return $sth->SUPER::execute_for_fetch(@_);
@@ -598,12 +618,12 @@ $DBD::ODBC::VERSION = '1.35';
                 foreach (@$tuple_batch_status) {
                     $tuple_count++ unless $_ == 7; # SQL_PARAM_UNUSED
                     next if ref($_);
-		    $_ = -1;	# we don't know individual row counts
+                    $_ = -1;	# we don't know individual row counts
+                }
+                if ($tuple_status) {
+                    push @$tuple_status, @$tuple_batch_status
+                        if defined($tuple_status);
                 }
-		if ($tuple_status) {
-		    push @$tuple_status, @$tuple_batch_status
-			if defined($tuple_status);
-		}
             }
             if (!defined($res)) {	# error
                 $row_count = undef;
@@ -630,7 +650,7 @@ DBD::ODBC - ODBC Driver for DBI
 
 =head1 VERSION
 
-This documentation refers to DBD::ODBC version 1.35.
+This documentation refers to DBD::ODBC version 1.39.
 
 =head1 SYNOPSIS
 
@@ -663,111 +683,6 @@ types with SQL_TIMESTAMP.  Also note that some tests may be skipped,
 such as t/09multi.t, if your driver doesn't seem to support returning
 multiple result sets.  This is normal.
 
-=head2 Version Control
-
-DBD::ODBC source code is under version control at svn.perl.org.  If
-you would like to use the "bleeding" edge version, you can get the
-latest from svn.perl.org via Subversion version control.  Note there
-is no guarantee that this version is any different than what you get
-from the tarball from CPAN, but it might be :)
-
-You may read about Subversion at L<http://subversion.tigris.org>
-
-You can get a subversion client from there and check dbd-odbc out via:
-
-   svn checkout http://svn.perl.org/modules/dbd-odbc/trunk <your directory name here>
-
-Which will pull all the files from the subversion trunk to your
-specified directory. If you want to see what has changed since the
-last release of DBD::ODBC read the Changes file or use "svn log" to
-get a list of checked in changes.
-
-=head2 Contributing
-
-There are seven main ways you may help with the development and
-maintenance of this module:
-
-=over
-
-=item Submitting patches
-
-Please use Subversion (see above) to get the latest version of
-DBD::ODBC from the trunk and submit any patches against that.
-
-Please, before submitting a patch:
-
-   svn update
-   <try and included a test which demonstrates the fix/change working>
-   <test your patch>
-   svn diff > describe_my_diffs.patch
-
-and send the resulting file to me and cc the dbi-users@perl.org
-mailing list (if you are not a member - why not!).
-
-=item Reporting installs
-
-Install CPAN::Reporter and report you installations. This is easy to
-do - see L</CPAN Testers Reporting>.
-
-=item Report bugs
-
-If you find what you believe is a bug then enter it into the
-L<http://rt.cpan.org/Dist/Display.html?Name=DBD-ODBC> system. Where
-possible include code which reproduces the problem including any
-schema required and the versions of software you are using.
-
-If you are unsure whether you have found a bug report it anyway or
-post it to the dbi-users mailing list.
-
-=item pod comments and corrections
-
-If you find inaccuracies in the DBD::ODBC pod or have a comment which
-you think should be added then go to L<http://annocpan.org> and submit
-them there. I get an email for every comment added and will review
-each one and apply any changes to the documentation.
-
-=item Review DBD::ODBC
-
-Add your review of DBD::ODBC on L<http://cpanratings.perl.org>.
-
-If you are a member on ohloh then add your review or register your
-use of DBD::ODBC at L<http://www.ohloh.net/projects/perl_dbd_odbc>.
-
-=item submit test cases
-
-Most DBDs are built against a single client library for the database.
-
-Unlike other DBDs, DBD::ODBC works with many different ODBC drivers.
-Although they all should be written with regard to the ODBC
-specification drivers have bugs and in some places the specification is
-open to interpretation. As a result, when changes are applied to
-DBD::ODBC it is very easy to break something in one ODBC driver.
-
-What helps enormously to identify problems in the many combinations
-of DBD::ODBC and ODBC drivers is a large test suite. I would greatly
-appreciate any test cases and in particular any new test cases for
-databases other than MS SQL Server.
-
-=item Test DBD::ODBC
-
-I have a lot of problems deciding when to move a development release
-to an official release since I get few test reports for development
-releases. What often happens is I call for testers on various lists,
-get a few and then get inundated with requests to do an official
-release. Then I do an official release and loads of rts appear out of
-nowhere and the cycle starts again.
-
-DBD::ODBC by its very nature works with many ODBC Drivers and it is
-impossible for me to have and test them all (this differs from other
-DBDs). If you depend on DBD::ODBC you should be interested in new
-releases and if you send me your email address suggesting you are
-prepared to be part of the DBD::ODBC testing network I will credit you
-in the Changes file and perhaps the main DBD::ODBC file.
-
-
-
-=back
-
 =head2 DBI attribute handling
 
 If a DBI defined attribute is not mentioned here it behaves as per the
@@ -950,7 +865,7 @@ one you are working in.
 However, if you wrote code using DBD::ODBC 1.29 or earlier and knew
 DBD::ODBC bound varchar/longvarchar columns as SQL_CHARs and decoded
 them yourself the new behaviour will adversely affect you (sorry). To
-revret to the old behaviour set odbc_old_unicode to true.
+revert to the old behaviour set odbc_old_unicode to true.
 
 You can also set this attribute in the attributes passed to the
 prepare method.
@@ -1221,6 +1136,8 @@ instead.  For example:
 
 See F<t/20SqlServer.t> for an example.
 
+In versions of SQL Server 2005 and later see "Multiple Active Statements (MAS)" in the DBD::ODBC::FAQ instead of using this attribute.
+
 =head3 odbc_has_unicode
 
 A read-only attribute signifying whether DBD::ODBC was built with the
@@ -1238,28 +1155,29 @@ When odbc_has_unicode is 1, DBD::ODBC will:
 =item bind columns the database declares as wide characters as SQL_Wxxx
 
 This means that UNICODE data stored in these columns will be returned
-to Perl in UTF-8 and with the UTF8 flag set.
+to Perl in UTF-8 and with the UTF-8 flag set.
 
 =item bind parameters the database declares as wide characters as SQL_Wxxx
 
-Parameters bound where the database declares the parameter as being
-a wide character (or where the parameter type is explicitly set to a
-wide type - SQL_Wxxx) can be UTF8 in Perl and will be mapped to UTF16
-before passing to the driver.
+Parameters bound where the database declares the parameter as being a
+wide character (or where the parameter type is explicitly set to a
+wide type - SQL_Wxxx) can be UTF-8 in Perl and will be mapped to
+UTF-16 before passing to the driver.
 
 =item SQL
 
-SQL passed to the C<prepare> or C<do> methods which has the UTF8 flag set
-will be converted to UTF16 before being passed to the ODBC APIs C<SQLPrepare>
-or C<SQLExecDirect>.
+SQL passed to the C<prepare> or C<do> methods which has the UTF-8 flag
+set will be converted to UTF-16 before being passed to the ODBC APIs
+C<SQLPrepare> or C<SQLExecDirect>.
 
 =item connection strings
 
 Connection strings passed to the C<connect> method will be converted
-to UTF16 before being passed to the ODBC API C<SQLDriverConnectW>. This happens
-irrespective of whether the UTF8 flag is set on the perl connect strings
-because unixODBC requires an application to call SQLDriverConnectW to indicate
-it will be calling the wide ODBC APIs.
+to UTF-16 before being passed to the ODBC API
+C<SQLDriverConnectW>. This happens irrespective of whether the UTF-8
+flag is set on the perl connect strings because unixODBC requires an
+application to call SQLDriverConnectW to indicate it will be calling
+the wide ODBC APIs.
 
 =back
 
@@ -1274,9 +1192,6 @@ platforms the WITH_UNICODE macro is B<not> enabled by default and to enable
 you need to specify the -u argument to Makefile.PL. Please bare in mind
 that some ODBC drivers do not support SQL_Wxxx columns or parameters.
 
-NOTE: Unicode support on Windows 64 bit platforms is currently
-untested.  Let me know how you get on with it.
-
 UNICODE support in ODBC Drivers differs considerably. Please read the
 README.unicode file for further details.
 
@@ -1335,14 +1250,122 @@ Bare in mind the bigger you set this the more memory DBD::ODBC will need
 to allocate when running execute_for_fetch and the memory required is
 max_length_of_pn * odbc_batch_size * n_parameters.
 
-=head3 odbc_disable_array_operations
+=head3 odbc_array_operations
+
+NOTE: this was briefly odbc_disable_array_operations in 1.35 and 1.36_1.
+I did warn it was experimental and it turned out the default was too
+ambitious and it was a poor name anyway. Also the default was to use
+array operations and now the default is the opposite.
+
+If set to true DBD::ODBC uses its own internal execute_for_fetch
+instead of DBI's default execute_for_fetch. The default is false.
+Using the internal execute_for_fetch should be quite a bit faster when
+using arrays of parameters for insert/update/delete operations as
+batches of parameters are sent to the database in one go. However,
+the required support in some ODBC drivers is a little sketchy and there
+is no way for DBD::ODBC to ascertain this until it is too late.
 
 Please read the documentation on L<execute_array> and L<execute_for_fetch>
 which details subtle differences in DBD::ODBC's implementation compared
 with using DBI's default implementation. If these difference cause you
-a problem you can set odbc_disable_array_operations to true and DBD::ODBC
+a problem you can set odbc_array_operations to false and DBD::ODBC
 will revert to DBI's implementations of the array methods.
 
+You can use the environment variable ODBC_DISABLE_ARRAY_OPERATIONS to
+switch array operations on/off too. When set to 1 array operations are
+disabled. When not set the default is used (which currently is off).
+When set to 0 array operations are used no matter what. I know this is
+slightly counter intuitive but I've found it difficult to change the
+name (it got picked up and used in a few places very quickly).
+
+=head3 odbc_taf_callback
+
+NOTE: this is experimental until I at least see more than one ODBC
+driver which supports TAF.
+
+Transparent Application Failover (TAF) is a feature in OCI that
+allows for clients to automatically reconnect to an instance in the
+event of a failure of the instance. The reconnect happens
+automatically from within the OCI (Oracle Call Interface)
+library.
+
+TAF supports a callback function which once registered is called
+by the driver to let you know what is happening and which allows
+you to a degree, to control how the failover is handled.
+
+You need to set up TAF on your instance first and that process
+is beyond the scope of this document. Once TAF is enabled you simply
+set C<odbc_taf_callback> to a code reference which should look like
+this:
+
+  sub taf_handler {
+   my ($dbh, $event, $type) = @_;
+   # do something here
+  }
+
+DBD::ODBC will pass the connection handle ($dbh), the Oracle event
+type (OCI_FO_END, OCI_FO_ABORT, OCI_FO_REAUTH, OCI_FO_BEGIN,
+OCI_FO_ERROR) and the Oracle type (OCI_FO_NONE, OCI_FO_SESSION,
+OCI_FO_SELECT, OCI_FO_TXNAL).  Consult the Oracle documentation for
+what these are. You can import these constants using the :taf export
+tag. If your instance is not TAF enabled it is likely an attempt to
+register a callback will fail but this is driver dependent (all
+DBD::ODBC does is make a SQLSetConnectAttr call and provide a C
+wrapper which calls your Perl subroutine).
+
+Here is a commented example:
+
+  my $h = DBI->connect('dbi:ODBC:oracle','xxx','yyy',
+                       {RaiseError => 1,
+                        odbc_taf_callback => \&taf_handler}) or die "connect";
+  while (1) {
+      my $s = $h->selectall_arrayref(q/select 1 from dual/);
+      sleep 5;
+  }
+
+  sub taf_handler {
+     my ($dbh, $event, $type) = @_;
+
+     #print "taf_handler $dbh, $event, $type\n";
+
+     if ($event == OCI_FO_BEGIN) {
+         print "Instance unavailable, stand by\n";
+         print "Your TAF type is : ",
+             ($type == OCI_FO_NONE ? "NONE" :
+                  ($type == OCI_FO_SESSION ? "SESSION" :
+                       ($type == OCI_FO_SELECT ? "SELECT" : "?"))) , "\n";
+         # start a counter and each time OCI_FO_ERROR is passed in we will
+         # count down and abort the failover when we hit 0.
+         $count = 10;
+         return 0;
+     } elsif ($event == OCI_FO_ERROR) {
+         # We get an OCI_FO_ERROR each time the failover fails
+         # sleep a while until the count hits 0
+         if (--$count < 1) {
+             print "Giving up\n";
+             return 0;            # give up
+         } else {
+             print "Retrying...\n";
+             sleep 1;
+             return OCI_FO_RETRY; # tell Oracle to retry
+         }
+     } elsif ($event == OCI_FO_REAUTH) {
+         print "Failed over user. Resuming Services\n";
+     } elsif ($event == OCI_FO_END) {
+         print "Failover ended - resuming\n";
+     }
+     return 0;
+  }
+
+NOTE: The above example is for use with the Easysoft Oracle ODBC
+Driver. ODBC does not define any standard way of supporting TAF and so
+different drivers may use different connection attributes to set it up
+or may even pass the callback different arguments. Unfortunately, I
+don't have access to any other ODBC driver which supports TAF. Until I
+see others I cannot create a generic interface. I'll happily accept
+patches for any other driver or if you send me a working copy of the
+driver and the documentation I will add support for it.
+
 =head2 Private statement attributes
 
 =head3 odbc_more_results
@@ -1358,7 +1381,7 @@ available.  SQL Server supports this feature.  Use this as follows:
   } while ($sth->{odbc_more_results});
 
 Note that with multiple result sets and output parameters (i.e,. using
-bind_param_inout), don't expect output parameters to be bound until ALL
+bind_param_inout), don't expect output parameters to written to until ALL
 result sets have been retrieved.
 
 =head2 Private statement methods
@@ -1377,13 +1400,13 @@ future e.g., it may support automatic concatenation of the lob
 parts onto the end of the C<$lob> with the addition of an extra flag
 or destination offset as in DBI's undocumented blob_read.
 
-The type the lob is retrieved as may be overriden in C<%attr> using
-C<TYPE =E<gt> sql_type>. C<%attr> is optional and if omitted defaults to
-SQL_C_BINARY for binary columns and SQL_C_CHAR/SQL_C_WCHAR for other
-column types depending on whether DBD::ODBC is built with unicode
-support. C<$chrs_or_bytes_read> will by the bytes read when the column
-types SQL_C_CHAR or SQL_C_BINARY are used and characters read if the
-column type is SQL_C_WCHAR.
+The type the lob is retrieved as may be overridden in C<%attr> using
+C<TYPE =E<gt> sql_type>. C<%attr> is optional and if omitted defaults
+to SQL_C_BINARY for binary columns and SQL_C_CHAR/SQL_C_WCHAR for
+other column types depending on whether DBD::ODBC is built with
+unicode support. C<$chrs_or_bytes_read> will by the bytes read when
+the column types SQL_C_CHAR or SQL_C_BINARY are used and characters
+read if the column type is SQL_C_WCHAR.
 
 When built with unicode support C<$length> specifes the amount of
 buffer space to be used when retrieving the lob data but as it is
@@ -1411,7 +1434,11 @@ the last (or only) column in your select list.
 
 NOTE: You can retrieve only part of a lob but you will probably have
 to call finish on the statement handle before you do anything else
-with that statement.
+with that statement. When only retrieving part of a large lob you
+could see a small delay when you call finish as some protocols used
+by ODBC drivers send the lob down the socket synchonously and there is
+no way to stop it (this means the ODBC driver needs to read all the
+lob from the socket even though you never retrieved it all yourself).
 
 NOTE: If your select contains multiple lobs you cannot read part of
 the first lob, the second lob then return to the first lob. You must
@@ -1562,7 +1589,7 @@ NAME, TYPE, PRECISION, SCALE, NULLABLE etc).>
 See the ODBC specification for the SQLColAttributes API.
 You call SQLColAttributes like this:
 
-  $dbh->func($column, $ftype, "ColAttributes");
+  $sth->func($column, $ftype, "ColAttributes");
 
   SQL_COLUMN_COUNT = 0
   SQL_COLUMN_NAME = 1
@@ -1592,12 +1619,12 @@ use DBI's NAME and NAME_xx attributes for portability.
 =head3 DescribeCol
 
 B<This private function is now superceded by DBI's statement attributes
-NAME, TYPE, PRECISION, SCLARE, NULLABLE etc).>
+NAME, TYPE, PRECISION, SCALE, NULLABLE etc).>
 
 See the ODBC specification for the SQLDescribeCol API.
 You call SQLDescribeCol like this:
 
-  @info = $dbh->func($column, "DescribeCol");
+  @info = $sth->func($column, "DescribeCol");
 
 The returned array contains the column attributes in the order described
 in the ODBC specification for SQLDescribeCol.
@@ -1617,13 +1644,16 @@ DiscardString to the prepare attributes then if the returned bound
 data is capable of being converted to that type the scalar's pv (the
 string portion of a scalar) is cleared.
 
+NOTE: post DBD::ODBC 1.37, DBD::ODBC binds all SQL_INTEGER columns as
+SQL_C_LONG and DiscardString is irrelevant.
+
 This is especially useful if you are using a module which uses a
-scalars flags and/or pv to decide if a scalar is a number. JSON::XS
+scalar's flags and/or pv to decide if a scalar is a number. JSON::XS
 does this and without this flag you have to add 0 to all bound column
 data returning numbers to get JSON::XS to encode it is N instead of
 "N".
 
-NOTE: For StrictlyTyped you need at least DBI 1.611.
+NOTE: For DiscardString you need at least DBI 1.611.
 
 =head3 StrictlyTyped
 
@@ -1640,7 +1670,7 @@ an SQL_INTEGER and the data is not able to be converted to an integer
 the ODBC driver will problably return "Invalid character value for
 cast specification (SQL-22018)".
 
-NOTE: For DiscardString you need at least DBI 1.611.
+NOTE: For StrictlyTyped you need at least DBI 1.611.
 
 =head3 TreatAsLOB
 
@@ -1813,7 +1843,7 @@ The DBI specification post 1.608 says in bind_param:
   The data type is 'sticky' in that bind values passed to execute()
   are bound with the data type specified by earlier bind_param()
   calls, if any.  Portable applications should not rely on being able
-  to change the data type after the first C<bind_param> call.
+  to change the data type after the first bind_param call.
 
 DBD::ODBC does allow a parameter to be rebound with another data type as
 ODBC inherently allows this. Therefore you can do:
@@ -1846,6 +1876,12 @@ when inserting, updating or deleting many rows in one go. Note,
 execute_array uses execute_for_fetch when the parameters are passed
 for column-wise binding.
 
+NOTE: DBD::ODBC 1.34_1 to DBD::ODBC 1.36_1 set the default to use
+DBD::ODBC's own execute_for_fetch but quite a few ODBC drivers just
+cannot handle it. As such, from DBD::ODBC 1.36_2 the default was
+changed to not use DBD::ODBC's execute_for_fetch (i.e., you need to
+enable it with odbc_array_operations).
+
 However, there are a small number of differences between using
 DBD::ODBC's execute_for_fetch compared with using DBI's default
 implementation (which simply calls execute repeatedly once per row).
@@ -1877,8 +1913,8 @@ one. Also, when more than one diagnostic is found for a row DBD::ODBC
 picks the first one (which is usually most relevant) as there is no
 way to report more than one diagnostic per row in the tuple_status. If
 the first problem of SQL_DIAG_ROW_NUMBER proves to be a problem for
-you the DBD::ODBC tracing will show all errors and if that is still
-not sufficient I'd consider adding a method to show all diagnostics.
+you the DBD::ODBC tracing will show all errors and you can also use
+L</odbc_getdiagrec> yourself.
 
 o Binding parameters with execute_array and execute_for_fetch does not
 allow the parameter types to be set. However, as parameter types are sticky
@@ -1893,14 +1929,57 @@ DBD::ODBC uses the ODBC API SQLPutData (see L</odbc_putdata_start>)
 which does not require large amounts of memory as large columns are
 sent in pieces.
 
+o A lot of drivers have bugs with arrays of parameters (see the ODBC
+FAQ). e.g., as of 18-MAR-2012 I've seen the latest SQLite ODBC driver
+seg fault and freeTDS 8/0.91 returns the wrong row count for batches.
+
+o B<DO NOT> attempt to do an insert/update/delete and a select in the
+same SQL with execute_array e.g.,
+
+  SET IDENTITY_INSERT mytable ON
+  insert into mytable (id, name) values (?,?)
+  SET IDENTITY_INSERT mytable OFF
+  SELECT SCOPE_IDENTITY()
+
+It just won't/can't work although you may not have noticed when using
+DBI's inbuilt execute_* methods. See rt 75687.
+
 =head3 type_info_all
 
 Many ODBC drivers now return 20 columns in type_info_all rather than
-the 19 DBI documents DBI documents. The 20th column is usually called
+the 19 DBI documents. The 20th column is usually called
 "USERTYPE".  Recent MS SQL Server ODBC drivers do this. Fortunately
-this should adversely affect you so long as you are using the keys
+this should not adversely affect you so long as you are using the keys
 provided at the start of type_info_all.
 
+=head3 Binding Columns
+
+The DBI specification allows a column type to be overriden in the call
+to the bind_col method. Mostly, DBD::ODBC ignores this type as it
+binds integers (SQL_INTEGER) as a SQL_C_LONG (since DBD::ODBC 1.38_1)
+and all other columns as SQL_C_CHAR or SQL_C_WCHAR and it is too late
+to change the bind type after the result-set has been described
+anyway. The only time when the TYPE passed to bind_col is used in
+DBD::ODBC is when it is SQL_NUMERIC or SQL_DOUBLE in which case
+DBD::ODBC will call DBI's sql_type_cast method.
+
+Since DBD::ODBC 1.38_1 if you attempt to change the bind type after
+the column has already bound DBD::ODBC will issue a warning and ignore
+your column type change e.g.,
+
+  my $s = $h->prepare(q/select a from mytable);
+  $s->execute;  # The column type was determined here
+  my $r;
+  $s->bind_col(1, \$r); # and bound as the right type here
+  $s->execute;
+  $s->bind_col(1, \$r, {TYPE => SQL_XXX}); # warning, type changed
+
+Basically, if you are passing a TYPE to bind_col with DBD::ODBC (other
+than SQL_NUMERIC or SQL_DOUBLE) your code is probably wrong.
+
+Significant changes occurred in DBD::ODBC at 1.38_1 for binding
+columns. Please see the Changes file.
+
 =head2 Unicode
 
 The ODBC specification supports wide character versions (a postfix of
@@ -1995,16 +2074,20 @@ Unicode column names are returned.
 
 If the DBMS reports the column as being a wide character (SQL_Wxxx) it
 will be bound as a wide character and any returned data will be
-converted from UTF16 to UTF8 and the UTF8 flag will then be set on the
-data.
+converted from UTF-16 to UTF-8 and the UTF-8 flag will then be set on
+the data.
 
 =item bound parameters
 
-If the perl scalars you bind to parameters are marked UTF8 and the
+If the perl scalars you bind to parameters are marked UTF-8 and the
 DBMS reports the type as being a wide type or you bind the parameter
 as a wide type they will be converted to wide characters and bound as
 such.
 
+=item metadata calls like table_info, column_info
+
+As of DBD::ODBC 1.32_3 meta data calls accept Unicode strings.
+
 =back
 
 Since version 1.16_4, the default parameter bind type is SQL_WVARCHAR
@@ -2025,12 +2108,6 @@ You cannot use unicode parameter names e.g.,
 You cannot use unicode strings in calls to prepare if you set the
 odbc_execdirect attribute.
 
-You cannot pass unicode catalog, schema, table and column names into
-metadata calls (like column_info) at present because the XS interface
-uses char * instead of Perl scalars. NOTE: as of DBD::ODBC 1.32_3 Unicode
-is fully supported for input parameters in table_info. As of DBD::ODBC
-1.32_5 Unicode is fully supported for input parameters in column_info.
-
 You cannot use the iODBC driver manager with DBD::ODBC built for
 unicode.
 
@@ -2052,7 +2129,7 @@ wchar_t types (which are usually 4) and hence DBD::ODBC will not work
 iODBC when built for unicode.
 
 The ODBC Driver must expect Unicode data specified in SQLBindParameter
-and SQLBindCol to be UTF16 in local endianness. Similarly, in calls to
+and SQLBindCol to be UTF-16 in local endianness. Similarly, in calls to
 SQLPrepareW, SQLDescribeColW and SQLDriverConnectW.
 
 You should be aware that once Unicode support is enabled it affects a
@@ -2062,7 +2139,7 @@ instance, when listing tables, columns etc some drivers
 even if the strings actually fit in 7-bit ASCII. As a result, there is
 an overhead for retrieving this column data as 2 bytes per character
 will be transmitted (compared with 1 when Unicode support is not
-enabled) and these strings will be converted into UTF8 but will end up
+enabled) and these strings will be converted into UTF-8 but will end up
 fitting (in most cases) into 7bit ASCII so a lot of conversion work
 has been performed for nothing. If you don't have Unicode table and
 column names or Unicode column data in your tables you are best
@@ -2082,9 +2159,33 @@ Wide characters returned from the ODBC driver will be converted to
 UTF-8 and the perl scalars will have the utf8 flag set (by using
 sv_utf8_decode).
 
-perl scalars which are UTF-8 and are sent through the ODBC API will be
+B<IMPORTANT>
+
+Perl scalars which are UTF-8 and are sent through the ODBC API will be
 converted to UTF-16 and passed to the ODBC wide APIs or signalled as
-SQL_WCHARs (e.g., in the case of bound columns).
+SQL_WCHARs (e.g., in the case of bound columns). Retrieved data which
+are wide characters are converted from UTF-16 to UTF-8. However, you
+should realise most ODBC drivers do not support UTF-16, ODBC only
+talks about wide characters being 2 bytes and UCS-2 and UCS-2 and
+UTF-16 are not the same. UCS-2 only supports Unicode characters in the
+first plane (the Basic Multilangual Plane or BMP) (code points U+0000
+to U+FFFF), the most frequently used characters. So why does DBD::ODBC
+currently encode in UTF-16? For around 97% of Unicode characters in
+the range 0-0xFFFF UCS-2 and UTF-16 are exactly the same (and where they
+differ there is no valid Unicode character as the range U+D800 to U+DFFF is
+reserved from use only as surrogate pairs). As the ODBC
+API currently uses UCS-2 it does not support Unicode characters with
+code points above 0xFFFF (if you know better I'd like to hear from
+you). However, because DBD::ODBC uses UTF-16 encoding you can still
+insert Unicode characters above 0xFFFF into your database and retrieve
+them back correctly but they will not being treated as a single
+Unicode character in your database e.g., a "select length(a_column)
+from table" with a single Unicode character above 0xFFFF will most
+likely return 2 and not 1 so you cannot use database functions on that
+data like upper/lower/length etc but you can at least save the data in
+your database and get it back. This is a fudge and I cannot say I'm
+overjoyed by it but it is what the majority of people who use
+DBD::ODBC have requested.
 
 When built for unicode, DBD::ODBC will always call SQLDriverConnectW
 (and not SQLDriverConnect) even if a) your connection string is not
@@ -2171,6 +2272,109 @@ SQL type to the end of the C<bind_param> method.
   $s = $h->prepare(q/insert into mytable values(?)/);
   $s->bind_param(1, "\x{263a}", SQL_WVARCHAR);
 
+=head2 Version Control
+
+DBD::ODBC source code is under version control at svn.perl.org.  If
+you would like to use the "bleeding" edge version, you can get the
+latest from svn.perl.org via Subversion version control.  Note there
+is no guarantee that this version is any different than what you get
+from the tarball from CPAN, but it might be :)
+
+You may read about Subversion at L<http://subversion.tigris.org>
+
+You can get a subversion client from there and check dbd-odbc out via:
+
+   svn checkout http://svn.perl.org/modules/dbd-odbc/trunk <your directory name here>
+
+Which will pull all the files from the subversion trunk to your
+specified directory. If you want to see what has changed since the
+last release of DBD::ODBC read the Changes file or use "svn log" to
+get a list of checked in changes.
+
+=head2 Contributing
+
+There are seven main ways you may help with the development and
+maintenance of this module:
+
+=over
+
+=item Submitting patches
+
+Please use Subversion (see above) to get the latest version of
+DBD::ODBC from the trunk and submit any patches against that.
+
+Please, before submitting a patch:
+
+   svn update
+   <try and included a test which demonstrates the fix/change working>
+   <test your patch>
+   svn diff > describe_my_diffs.patch
+
+and send the resulting file to me and cc the dbi-users@perl.org
+mailing list (if you are not a member - why not!).
+
+=item Reporting installs
+
+Install CPAN::Reporter and report you installations. This is easy to
+do - see L</CPAN Testers Reporting>.
+
+=item Report bugs
+
+If you find what you believe is a bug then enter it into the
+L<http://rt.cpan.org/Dist/Display.html?Name=DBD-ODBC> system. Where
+possible include code which reproduces the problem including any
+schema required and the versions of software you are using.
+
+If you are unsure whether you have found a bug report it anyway or
+post it to the dbi-users mailing list.
+
+=item pod comments and corrections
+
+If you find inaccuracies in the DBD::ODBC pod or have a comment which
+you think should be added then go to L<http://annocpan.org> and submit
+them there. I get an email for every comment added and will review
+each one and apply any changes to the documentation.
+
+=item Review DBD::ODBC
+
+Add your review of DBD::ODBC on L<http://cpanratings.perl.org>.
+
+If you are a member on ohloh then add your review or register your
+use of DBD::ODBC at L<http://www.ohloh.net/projects/perl_dbd_odbc>.
+
+=item submit test cases
+
+Most DBDs are built against a single client library for the database.
+
+Unlike other DBDs, DBD::ODBC works with many different ODBC drivers.
+Although they all should be written with regard to the ODBC
+specification drivers have bugs and in some places the specification is
+open to interpretation. As a result, when changes are applied to
+DBD::ODBC it is very easy to break something in one ODBC driver.
+
+What helps enormously to identify problems in the many combinations
+of DBD::ODBC and ODBC drivers is a large test suite. I would greatly
+appreciate any test cases and in particular any new test cases for
+databases other than MS SQL Server.
+
+=item Test DBD::ODBC
+
+I have a lot of problems deciding when to move a development release
+to an official release since I get few test reports for development
+releases. What often happens is I call for testers on various lists,
+get a few and then get inundated with requests to do an official
+release. Then I do an official release and loads of rts appear out of
+nowhere and the cycle starts again.
+
+DBD::ODBC by its very nature works with many ODBC Drivers and it is
+impossible for me to have and test them all (this differs from other
+DBDs). If you depend on DBD::ODBC you should be interested in new
+releases and if you send me your email address suggesting you are
+prepared to be part of the DBD::ODBC testing network I will credit you
+in the Changes file and perhaps the main DBD::ODBC file.
+
+=back
+
 =head2 CPAN Testers Reporting
 
 Please, please, please (is that enough), consider installing
